
# q-expansion of newform 3450.2.a.e, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3450
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [0], [1], [-5], [3], [2], [-3], [1], [-1], [0], [4], [-3], [5], [-6], [0], [-6], [4], [-4], [-2], [9], [-7], [-1], [-2], [0], [6], [7], [-12], [-4], [-6], [-18], [12], [16], [-10], [-20], [18], [-4], [-4], [-12], [-9], [10], [-6], [-3], [10], [15], [-11], [-2], [2], [-24], [-8], [-11], [-8], [-2], [-12], [-22], [28], [-9], [-28], [-31], [-22], [-4], [-28], [-20], [-20], [32], [-13], [20], [8], [-20], [-15], [11], [-9], [37], [-22], [-20], [-9], [10], [2], [12], [17], [25], [6], [0], [-12], [20], [-20], [14], [18], [-37], [12], [-27], [17], [18], [0], [-34], [-11], [-2], [-4], [19], [-31], [6], [2], [11], [18], [36], [17], [2], [9], [-36], [26], [-14], [14], [10], [-36], [-25], [4], [-19], [24], [1], [45], [24], [27], [8], [-18], [-14], [-36], [-44], [30], [8], [-24], [24], [-5], [7], [28], [23], [6], [12], [-37], [-10], [39], [-2], [53], [2], [53], [-23], [25], [-49], [6], [6], [18], [22], [40], [24], [6], [-33], [-9], [4], [33], [60], [32], [32], [20], [-32], [-31], [24], [9], [-20], [49], [-8], [42], [1], [5], [57], [22], [14], [-36], [-34], [41], [9], [-12], [-44], [21], [-19], [-57], [29], [12], [26], [19], [-31], [-60], [23], [-46], [-45], [50], [-65], [14], [30], [34], [0], [-12], [-20], [-28], [13], [-46], [-41], [39], [20], [0], [-43], [62], [-21], [-35], [31], [-13], [-24], [-36], [-66], [38], [-21], [22], [-5], [-45], [30], [11], [50], [-1], [-6], [0], [32], [46], [-14], [72], [-43], [-13], [-32], [-63], [-50], [24], [8], [-54], [-69], [-6], [12], [-14], [2], [-20], [-8], [-60], [-5], [73], [12], [42], [-33], [12], [69], [6], [-50], [63], [-29], [-30], [-16], [47], [-50], [62], [2], [11], [-42], [28], [29], [75], [-67], [-52], [-22], [49], [-63], [70], [-12], [-29], [-30], [52], [22], [32], [-11], [-34], [2], [-24], [74], [-12], [62], [-61], [-18], [53], [71], [4], [-10], [-70], [73], [57], [-42], [-28], [-62], [62], [1], [40], [-20], [-33], [54], [72], [-4], [-25], [-40], [-18], [4], [-80], [72], [32], [0], [-90], [-14], [-46], [28], [-18], [33], [34], [18], [-62], [12], [-66], [51], [-42], [-78], [-27], [82], [-18], [52], [-4], [24], [38], [27], [-32], [10], [92], [-15], [88], [-32], [78], [18], [34], [21], [38], [38], [10], [-57], [-16], [-22], [-37], [-9], [39], [47], [4], [40], [75], [72], [-67], [-52], [-86], [-2], [-16], [56], [14], [-88], [66], [30], [-31], [-25], [-34], [23], [-20], [-82], [-85], [46], [-54], [54], [-62], [-93], [84], [18], [-10], [-80], [8], [-100], [-41], [-72], [22], [-72], [0], [83], [42], [69], [-54], [-1], [-58], [-3], [30], [62], [-76], [36], [22], [15], [46], [-14], [-3], [-13], [73], [-32], [-41], [10], [-80], [88], [-11], [44], [22], [18], [10], [77], [-19], [13], [33], [22], [-26]]
